#include <SDL/SDL.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

/* constante pentru rezolutia grafica */
#define SCREEN_WIDTH 640
#define SCREEN_HEIGHT 480

#define DIM_PIESA 4

unsigned short piese[] = { 0x0E80, 0x0E20, 0x0E40,
  0x0660, 0x4444, 0x06C0, 0x0C60
};

unsigned short roteste(unsigned short cod) {
  unsigned short cod_nou;
  cod_nou = (cod & 1 << 12) >> 12 |
      (cod & 1 << 8) >> 7 |
      (cod & 1 << 4) >> 2 |
      (cod & 1 << 3) |
      (cod & 1 << 13) >> 9 |
      (cod & 1 << 9) >> 4 |
      (cod & 1 << 5) << 1 |
      (cod & 1 << 1) << 6 |
      (cod & 1 << 14) >> 6 |
      (cod & 1 << 10) >> 1 |
      (cod & 1 << 6) << 4 |
      (cod & 1 << 2) << 9 |
      (cod & 1 << 15) >> 3 |
      (cod & 1 << 11) << 2 |
      (cod & 1 << 7) << 7 | (cod & 1 << 3) << 12;
  return cod_nou;
}

void deseneaza_piesa(SDL_Surface * screen, unsigned short cod) {
  SDL_Rect r;
  Uint32 culoare;
  int i, j;
  unsigned short masca =
      1 << (8 * sizeof(unsigned short) - 1);
  r.w = r.h = 32;
  r.y = 100;
  for (i = 0; i < DIM_PIESA; i++) {
    r.x = 100;
    for (j = 0; j < DIM_PIESA; j++) {
      if (cod & masca)
        culoare =
            SDL_MapRGB(screen->format, 0xFF, 0x80, 0x40);
      else
        culoare =
            SDL_MapRGB(screen->format, 0x00, 0x00, 0x00);
      SDL_FillRect(screen, &r, culoare);
      r.x += r.w;
      masca >>= 1;
    }
    r.y += r.h;
  }
}

int main(int argc, char **argv) {
  /* initializare generator numere aleatoare */
  srand(time(NULL));
  /* initializare SDL */
  if (SDL_Init(SDL_INIT_VIDEO) == -1) {
    printf("Failed to initialize SDL: %s.\n", SDL_GetError());
    exit(EXIT_FAILURE);
  }
  atexit(SDL_Quit);
  /* configurare mod video */
  SDL_Surface *screen =
      SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, 8,
                       SDL_SWSURFACE);
  if (screen == NULL) {
    printf("Failed to set video mode: %s.\n", SDL_GetError());
    exit(EXIT_FAILURE);
  }

  /* bucla principala de program:
     - desenam piesa curenta de Tetris
     - pana cand utilizatorul apasa ESC 
     - cu sagetile STANGA si DREAPTA se schimba
     piesa curenta */
  SDL_Event event;
  int gata = 0, idx = 0;
  int npiese = sizeof(piese) / sizeof(piese[0]);
  unsigned short cod = piese[idx];
  while (!gata) {
    /* desenam pe suprafata grafica din memorie */
    deseneaza_piesa(screen, cod);
    /* afisam suprafata grafica pe ecran */
    SDL_Flip(screen);
    /* daca si cat timp utilizatorul apasa taste, 
       reactionam la ele */
    while (SDL_PollEvent(&event)) {
      switch (event.type) {
        /* daca e apasare de tasta */
      case SDL_KEYDOWN:
        switch (event.key.keysym.sym) {
        case SDLK_ESCAPE:
          gata = 1;
          break;
        case SDLK_LEFT:
          idx--;
          if (idx < 0)
            idx = npiese - 1;
          cod = piese[idx];
          break;
        case SDLK_RIGHT:
          idx++;
          if (idx >= npiese)
            idx = 0;
          cod = piese[idx];
          break;
        case SDLK_DOWN:
          cod = roteste(cod);
          break;
        default:
          break;
        }
        break;
        /* daca e comanda de iesire din program */
      case SDL_QUIT:
        gata = 1;
        break;
      }
    }
  }
  SDL_Quit();
  exit(EXIT_SUCCESS);
}
